<?php

namespace Drupal\pid_field_set\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'Configurable' formatter.
 *
 * @FieldFormatter(
 *   id = "pid_field_set_config",
 *   label = @Translation("Configurable Formatter"),
 *   field_types = {
 *     "pid_field_set"
 *   }
 * )
 */


class PIDFieldSetConfigFormatter extends \Drupal\Core\Field\FormatterBase
{

  /**
   * @inheritDoc
   */
  public function viewElements(FieldItemListInterface $items, $langcode)
  {
    $element = [];

    $this->sanitizeSettings();

    foreach ($items as $delta => $item) {
      $values = $item->getValue();
      $element[$delta] = [
        '#theme' => 'pid_field_set_config',
        "#attributes" => [
          'class' => [
            'pid_field+_set'
          ],
        ],
      ];

      $element[$delta]["#pid_type"] = $values['pid_type'];
      $element[$delta]["#persistent_item"] = $values['persistent_item'];
      $element[$delta]["#pid_url"] = $values['pid_url'];
      $element[$delta]["#url_prefix"] = $values['url_prefix'];

      $element[$delta]["#show_id_as_url"] = $this->getSetting('showIDAsUrl');
      $element[$delta]["#id_open_link_in"] = $this->getSetting('IDOpenLinkIn');
      $element[$delta]["#show_pid__url"] = $this->getSetting('showPIDUrl');
    }

    return $element;
  }

  public static function defaultSettings()
  {
    return array_merge(
      [
        'showIDAsUrl' => TRUE,
        'IDOpenLinkIn' => '_blank',
        'showPIDUrl' => TRUE,
      ],
      parent::defaultSettings());
  }

  protected static function getOpenLinkInValues(): array
  {
    return [
      '_self' => t('The same tab/window'),
      '_blank' => t('A new tab/window'),
    ];
  }

  public function settingsSummary()
  {
    // Sanitize current settings.
    $this->sanitizeSettings();

    // Summarize.
    $summary = [];
    if ($this->getSetting('showIDAsUrl'))
      $summary[] = $this->t('Show ID as URL Link');
    else
      $summary[] = $this->t('Show ID as Plain Text');

    if ($this->getSetting('showIDAsUrl'))
    {
      switch ($this->getSetting('IDOpenLinkIn'))
      {
        case '_self':
          $summary[] = $this->t('ID URL: Open in current tab/window');
          break;

        case '_blank':
          $summary[] = $this->t('ID URL: Open in new tab/window');
          break;

      }
    }
    if ($this->getSetting('showPIDUrl'))
      $summary[] = $this->t('Show ID URL');
    else
      $summary[] = $this->t('Hide ID URL');

    return $summary;
  }

  public function settingsForm(array $form, FormStateInterface $formState)
  {
    $this->sanitizeSettings();

    $weight = 0;
    $marker = rand();
    $elements = [];

    $elements['showIDAsUrl'] = [
      '#title' => $this->t('Show Persistent ID as URL Link'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('showIDAsUrl'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
      '#attributes' => [
        'class' => [
          'showIDUrl-' . $marker,
        ],
      ],
    ];

    $elements['IDOpenLinkIn'] = [
      '#title' => $this->t('Open link in'),
      '#type' => 'select',
      '#options' => $this->getOpenLinkInValues(),
      '#default_value' => $this->getSetting('IDOpenLinkIn'),
      '#weight' => $weight++,
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
      ]
    ];

    $elements['showPIDUrl'] = [
      '#title' => $this->t('Show Persistent ID URL'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('showPIDUrl'),
      '#weight' => $weight++,
    ];

    return $elements;
  }


  protected function sanitizeSettings()
  {
    $showIDAsUrl = $this->getSetting('showIDAsUrl');
    $IDOpenLinkIn = $this->getSetting('IDOpenLinkIn');
    $showPIDUrl = $this->getSetting('showPIDUrl');

    $this->setSetting('showIDAsUrl', boolval($showIDAsUrl));
    $this->setSetting('IDOpenLinkIn', $IDOpenLinkIn);
    $this->setSetting('showPIDUrl', boolval($showPIDUrl));
  }
}
