<?php


namespace Drupal\pid_field_set\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Plugin implementation of the 'sample_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "pid_field_set_widget",
 *   module = "pid_field_set",
 *   label = @Translation("Pid Field Set Widget"),
 *   field_types = {
 *     "pid_field_set"
 *   }
 * )
 */

class PidFieldSetFieldWidget extends WidgetBase
{
  private $DOI = 'DOI';
  private $ARK = 'ARK';
  private $Handle = 'Handle';
  private $UUID = 'UUID';
  private string $url = '';


  public function getIsHidden($key): bool
  {
    return false;
  }

  //Get uuid of the entity and use that to construct an URL
  private function setURL($entity)
  {
    if (empty($this->url) && ($entity instanceof \Drupal\node\NodeInterface)) {
      $base_url = \Drupal::request()->getSchemeAndHttpHost();
      //$this->url = $base_url . '/uuid/' . $entity->uuid();
      $this->url = '#';
    }
  }

  private function setURLFromFormState(FormStateInterface $form_state)
  {
    $entity = $form_state->getFormObject()->getEntity();
    //if ($entity instanceof \Drupal\node\NodeInterface)
    //if (strcmp($entity->bundle(), 'pid_field_set') == 0)
    $this->setURL($entity);
  }

  //Mint a new persistent identifier
  public function onMint($entity, $pid_type)
  {
    $this->setURL($entity);

    if (strcmp($pid_type, $this->DOI) == 0) {
      return \Drupal::service('doi_datacite.minter.datacitedois')->mintDraft($entity);
    } else if (strcmp($pid_type, $this->ARK) == 0) {
      return \Drupal::service('ezid.minter.ezid')->mint($entity);
    } else if (strcmp($pid_type, $this->Handle) == 0) {
      return \Drupal::service('hdl.minter.hdl')->mint($entity, $this->url);
    } else if (strcmp($pid_type, $this->UUID) == 0) {
      return \Drupal::service('persistent_identifiers.minter.uuid')->mint($entity, $this->url); ;
    }
    return '';
  }

  //Should the Provision button be hidden? config of the widget decides this
  public function shouldHideProvisionButton(): bool
  {
    if ($this->getSetting('hidePersistentMinterButton'))
      return true;
    return false;
  }

  public function shouldHideExistingPIDURL(): bool
  {
    if ($this->getSetting('hideExistingPIDURL'))
      return true;
    return false;
  }

  //Is the config setup for a particular persistent identifier type complete? If no, will not show the Provision button for that pid type
  public function isConfigComplete($pid_type): bool
  {
    if (strcmp($pid_type, $this->DOI) == 0) {
      $config = \Drupal::config('doi_datacite.settings');
      $api_endpoint = $config->get('doi_datacite_api_endpoint');
      $doi_prefix = $config->get('doi_datacite_prefix');
      $doi_suffix_source = $config->get('doi_datacite_suffix_source');
      $api_username = $config->get('doi_datacite_username');
      $api_password = $config->get('doi_datacite_password');

      if (empty($api_endpoint) || empty($doi_prefix) || empty($doi_suffix_source) || empty($api_username) || empty($api_password)) {
        return false;
      }
    }
    elseif (strcmp($pid_type, $this->ARK) == 0) {
      $config = \Drupal::config('ezid.settings');
      $ezid_user = $config->get('ezid_user');
      $password = $config->get('ezid_password');
      $ezid_api_endpoint = $config->get('ezid_api_endpoint');
      $ezid_shoulder = $config->get('ezid_shoulder');

      if(empty($ezid_user) || empty($password) || empty($ezid_api_endpoint) || empty($ezid_shoulder))
        return false;
    }
    elseif (strcmp($pid_type, $this->Handle) == 0) {
      $config = \Drupal::config('hdl.settings');
      $endpoint_url = $config->get('hdl_handle_api_endpoint');
      $admin_handle = $config->get('hdl_admin_handle');
      $handle_admin_index = $config->get('hdl_admin_index');
      $private_key_pem = $config->get('hdl_admin_private_key');

      if (empty($endpoint_url) || empty($admin_handle) || empty($handle_admin_index) || empty($private_key_pem))
        return false;
    }
    return true;
  }


  //DOI requires finalization.
  public function finalizeDOI($element, FormStateInterface $form_state, $value, $target_form_element, $pid_type): bool
  {
    //Call the hook that is provided by the entity that is using the pid_field_set module
    $result = \Drupal::moduleHandler()->invokeAll('pid_field_set_alter_DOI_finalization', array($element, $form_state));

    //Finalization information is indexed by uuid
    $uuid = $form_state->getFormObject()->getEntity()->uuid();

    if (!key_exists($uuid, $result)) {
      \Drupal::messenger()->addError($this->t("finalizeDOI: Missing needed information for finalization. Please provide necessary information through hook: pid_field_set_alter_DOI_finalization"));
      return false;
    }
    if (is_null($value) || empty($value))
    {
      \Drupal::messenger()->addError($this->t("finalizeDOI: You need to click 'Provision DOI' to get a DOI before you can finalize it"));
      return false;
    }
    $form_state->clearErrors();

    $pid = $value;
    $values = $result[$uuid];;
    $keys = array_keys($values);
    $count = count($keys);

    //Get 'title' informaton
    if (array_key_exists('title', $values))
      $title = $values['title'][0]['value'];
    else {
      $title = $form_state->getValue('title')[0]['value'];
      \Drupal::logger('persistent_fields')->info("finalizeDOI() title =  " . $title);
    }

    $url = null;
    if (array_key_exists('field_url', $values))
      $url = $values['field_url'][0]['value'];
    else {
      if (!empty($pid_type)) {
        if (array_key_exists('pid_url_' . $pid_type, $target_form_element))
          $url = $target_form_element['pid_url_' . $pid_type]['#value'];
      }
    }
    if (empty($url)) {

      $node = \Drupal::routeMatch()->getParameter('node');
      if ($node instanceof \Drupal\node\NodeInterface) {
        $nid = $node->id();
        if (!empty($nid))
          $url = \Drupal::request()->getSchemeAndHttpHost() . "/node/" . $nid;
      }
    }
    \Drupal::logger('persistent_fields')->info("finalizeDOI() url =  " . $url);
    $url_missing = False;
    if (empty($url)) {
      \Drupal::messenger()->addError(t('finalizeDOI: url is needed for finalization.'));
      $url_missing = True;
    }
    //Get 'publisher' information
    $publisher_missing = False;
    if (array_key_exists('field_publisher', $values))
      $publisher = $values['field_publisher'][0]['value'];
    else {
      $publisher_missing = True;
      \Drupal::messenger()->addError(t('finalizeDOI: Information about Publisher is needed for finalization.'));
    }

    //Get 'publication year' information
    $publication_year_missing = False;
    if (array_key_exists('field_publication_year', $values))
      $publication_year = $values['field_publication_year'][0]['value'];
    else {
      $publication_year_missing = True;
      \Drupal::messenger()->addError(t('finalizeDOI: Information about Publication Year is needed for finalization.'));
    }

    //Get 'publication type' information
    $type_of_publication_missing = False;
    if (array_key_exists('field_type_of_publication', $values))
      $resource_type = $values['field_type_of_publication'][0]['value'];
    else {
      $type_of_publication_missing = True;
      \Drupal::messenger()->addError(t('finalizeDOI: Information about Type of Publication is needed for finalization.'));
    }

    if ($url_missing || $publisher_missing || $publication_year_missing || $type_of_publication_missing)
      return false;

    $pid_response = [
      "doi_datacite_title" => $title,
      "doi_datacite_url" => $url,
      "doi_datacite_creator" => \Drupal::currentUser()->getAccountName(),
      "doi_datacite_publisher" => $publisher,
      "doi_datacite_publication_year" => $publication_year,
      "doi_datacite_resource_type" => $resource_type
    ];

    $pid_response = \Drupal::service('doi_datacite.minter.datacitedois')->save($pid, $pid_response);
    if (!is_null($pid_response))
      \Drupal::messenger()->addStatus(t('Your DOI ' . $pid . ' has been finalized.'));
    else {
      \Drupal::messenger()->addError(t('finalizeDOI: Error occurred while finalizing your DOI' . $pid));
      return false;
    }
    return true;
  }

  private function get_specified_value($element, FormStateInterface $form_state, $key) : string
  {
    $element_copy = $element['#parents'];
    //ksm($form_state);
    $ct = 0;
    foreach ($element_copy as $level) {
      if (str_starts_with($level, 'persistent_item_new') || str_starts_with($level, 'persistent_item'))
        $element_copy[$ct] = $key;
        //$element_copy[$ct] = 'pid_type';
      $ct++;
    }
    $val  = $form_state->getValue($element_copy);
    if (!empty($val))
      return $val;
    else
      return '';
  }

  //Do finalization for DOI. For other persistent identifier types. warn users finalization is not needed
  public function validate_new_value($element, FormStateInterface $form_state)
  {
    $this->setURLFromFormState($form_state);

    $triggering = $form_state->getTriggeringElement();
    $array_parents = $triggering['#array_parents'];
    $value = $element['#value'];
    $pid_type = $this->get_specified_value($element, $form_state, 'pid_type');
    $field_name = end($element['#parents']);

    if (str_starts_with($field_name, 'persistent_item_') && !str_ends_with($field_name, $pid_type)) {
      return;
    }

    \Drupal::logger('persistent_fields')->info("validate_new_value() is called and pid_type is " . $pid_type);

    //If users click 'Finalize and Save DOI' button, make sure the pid_type is DOI
    /*
    if ($array_parents === array('actions', 'finalize_DOI')) {
      if (strcmp($pid_type, 'DOI') != 0) {
        $form_state->setError($triggering, $pid_type . " persistent identifier doesn't require finalization. Please click Save instead.");
      }
      if (strlen($value) != 0 && strcmp($pid_type, 'DOI') == 0) {
        if (!$this->finalizeDOI($element, $form_state, $value))
          $form_state->setError($element, "Finalization of DOI:  " . $value . " failed");
      }
    }*/
  }

  //Validate the persistent identifier
  public function validate($element, FormStateInterface $form_state)
  {
    $this->setURLFromFormState($form_state);

    $triggering = $form_state->getTriggeringElement();
    $array_parents = $triggering['#array_parents'];

    $value = $element['#value'];
    $field_name = end($element['#parents']);

    $pid_type = $this->get_specified_value($element, $form_state, 'pid_type');
    \Drupal::logger('persistent_fields')->info("validate() is called and pid_type is " . $pid_type);
    $is_persistent_new = $this->get_specified_value($element, $form_state, 'persistent_new_checkbox');

    if ($is_persistent_new)
    {
      \Drupal::logger('persistent_fields')->info("validate() is called and is_persistent_new is: " . $is_persistent_new);
      return;
    }    

    if ($array_parents === array('actions', 'finalize_DOI')) {
      if (strcmp($pid_type, 'DOI') != 0) {
        $form_state->setError($triggering, $pid_type . " persistent identifier doesn't require finalization. Please click Save instead.");
      }
    }

    if (str_starts_with($field_name, 'persistent_item_') && !str_ends_with($field_name, $pid_type)) {
      return;
    }
    //No need to validate if the field is empty
    if (strlen($value) === 0) {
      return;
    }

    $title = $element['#title'];

    \Drupal::logger('persistent_fields')->info("validate() is called and title is " . $title);

    //If the persistent identifier is DOI
    if (strcmp($pid_type, "DOI") == 0) {
      $pattern = "/^10[.]/";
      if (!preg_match($pattern, $value)) {
        $form_state->setError($element, $this->t('The given DOI: ' . $value . ' is not valid.'));
      } else {
        $client = \Drupal::httpClient();
        try {
          $doi_url = 'https://doi.org/' . $value;
	      $request = $client->request('GET', $doi_url,
                  ['headers' => ['Accept' => 'text/x-bibliography; style=apa' ]]);
          //$request = $client->get($doi_url);

          $status = $request->getStatusCode();
          if ($status != 200) {
            $messages = $this->t('The DOI: ' . $value . ' is not found.');
            $form_state->setError($element, $messages);
          }
        } catch (RequestException $e) {
          \Drupal::logger('persistent_fields')->notice('exception = ' . $e->getMessage());

          $messages = $this->t('The DOI: ' . $value . ' is not found.');
          $form_state->setError($element, $messages);
        }
      }
    }

    //If the persistent identifier is ARK
    if (strcmp($pid_type, "ARK") == 0) {
      $client = \Drupal::httpClient();
      try {
        $params = '/??';
        $request_url = 'https://n2t.net/' . $value . $params;
        $request = $client->get($request_url);

        $status = $request->getStatusCode();
        if ($status != 200) {
          $messages = $this->t('The ARK, ' . $value . ' is not found.');
          $form_state->setError($element, $messages);
        }
      } catch (RequestException $e) {
        //An error happened.
        \Drupal::logger('persistent_fields')->notice('exception = ' . $e->getMessage());

        $messages = $this->t('The ARK: ' . $value . ' is not found.');
        $form_state->setError($element, $messages);
      }
    }

    //If the persistent identifier is Handle
    if (strcmp($pid_type, "Handle") == 0) {
      $client = \Drupal::httpClient();
      try {
        $params = '?type=URL&type=EMAIL';
        $request_url = 'https://hdl.handle.net/api/handles/' . $value . $params;
        $request = $client->get($request_url);
        $status = $request->getStatusCode();
        if ($status != 200) {
          $messages = $this->t('The Handle: ' . $value . ' is not found.');
          $form_state->setError($element, $messages);
        }
      } catch (RequestException $e) {
        //An error happened.
        \Drupal::logger('persistent_fields')->notice('exception = ' . $e->getMessage());
        $messages = $this->t('The Handle: ' . $value . ' is not found.');
        $form_state->setError($element, $messages);
      }
    }

    //UUID doesn't allow  existing value. So below codes won't get executed. Just keep them here.
    if (strcmp($pid_type, "UUID") == 0) {
      $client = \Drupal::httpClient();
      try {
        $request_url = $value;
        $request = $client->get($request_url);
        $status = $request->getStatusCode();
        if ($status != 200) {
          $messages = $this->t('The UUID: ' . $value . ' cannot be resolved.');
          $form_state->setError($element, $messages);
        }
      } catch (RequestException $e) {
        //An error happened.
        \Drupal::logger('persistent_fields')->notice('exception = ' . $e->getMessage());
        $messages = $this->t('The UUID: ' . $value . ' is not found.');
        $form_state->setError($element, $messages);
      }
    }

    //If 'Finalize and Save DOI' button is click, do finalization for DOI
    /*
    if ($array_parents === array('actions', 'finalize_DOI')) {
      \Drupal::logger('persistent_fields')->info("finalizeDOI() is called: ");
      if (strlen($value) != 0) {
        if (!$this->finalizeDOI($element, $form_state, $value))
          $form_state->setError($element, "Finalization of DOI:  " . $value . " failed");
      }
    }*/
  }

  public function finalizeCallback(array &$form, FormStateInterface $form_state)
  {
    $pid_type = "DOI";
    $element = $form_state->getTriggeringElement();
    $target_form_element = NestedArray::getValue($form, array_slice($element['#array_parents'], 0, -1));
    $value = $target_form_element['persistent_item_new_'.$pid_type]['#value'] ;

    $response = new AjaxResponse();
    if ($this->finalizeDOI($element, $form_state, $value, $target_form_element, $pid_type)) {
      $target_form_element['persistent_minter_button_'.$pid_type]['#attributes']['disabled'] = True;
      $target_form_element['persistent_finalize_button_'.$pid_type]['#attributes']['disabled'] = True;
      $target_form_element['persistent_item_new_' . $pid_type]['#title'] = "New Persistent Identifier (finalized)";
      $target_form_element['doi_finalized']['#value'] = 'yes';

      return $target_form_element;
    }
    else {
      $error_msgs = \Drupal::messenger()->messagesByType('error');
      $response->addCommand(new MessageCommand(t("Failed to finalize DOI $value")));
      foreach ($error_msgs  as $error_msg) {
        if (str_starts_with($error_msg, "finalizeDOI: "))
          $response->addCommand(new MessageCommand($error_msg));
      }
      return $response;
    }
    /*
    $wrap_id = $target_form_element['#attributes']['id'];
    \Drupal::logger('persistent_fields')->info(t("The wrap_id is:  @wrap_id", ['@wrap_id' => $wrap_id]));
    $response->addCommand(new ReplaceCommand('#'.$wrap_id, $target_form_element));
    return $response;
    */
  }

  //This is the callback function that is called when users click "Provision" button
  public function myAjaxCallback(array &$form, FormStateInterface $form_state)
  {
    \Drupal::logger('persistent_fields')->info("myAjaxCallback() is called: ");

    $entity = $form_state->getFormObject()->getEntity();
    $triggering = $form_state->getTriggeringElement();
    $field_name = $triggering['#array_parents'][0];
    $value = $form_state->getValue($triggering['#parents']);


    \Drupal::logger('persistent_fields')->info(t("The field name is:  @field_name, @value", ['@field_name' => $field_name, '@value' => $value]));
    $persistent_item_field = $entity->get($field_name);

    $target_form_element = NestedArray::getValue($form, array_slice($triggering['#array_parents'], 0, -1));

    $pid_type = $target_form_element['pid_type']['#value'];

    if ($value) {
      $pid = $this->onMint($entity, $pid_type);

      \Drupal::logger('persistent_fields')->info(t("Persistent identifier created for node @pid.", ['@pid' => $pid]));


      if (is_null($pid)) {
        \Drupal::logger('persistent_fields')->warning(t("Persistent identifier not created for node @nid.", ['@nid' => $entity->id()]));
        \Drupal::messenger()->addWarning(t("Problem creating persistent identifier for this node. Details are available in the Drupal system log."));
        return $target_form_element;
      }

      $target_form_element['persistent_item_new_'.$pid_type]['#value'] = $pid;
      $target_form_element['persistent_new_checkbox']['#attributes']['checked'] = 'checked';
      $target_form_element['persistent_minter_button_'.$pid_type]['#attributes']['disabled'] = True;
      $target_form_element['pid_type']['#default_value'] = $pid_type;
      $target_form_element['pid_type']['#value'] = $pid_type;

      //Make sure the 'Provision' button is disabled once it is clicked
      $fields_to_buttons = $this->get_pid_buttons();
      foreach ($this->get_pid_fields() as $item) {
        if (!empty($target_form_element[$item]['#value'])) {
          $target_form_element[$fields_to_buttons[$item]]['#attributes']['disabled'] = True;
        }
      }
    }

    return $target_form_element;
  }

  private function get_pid_types(): array
  {
    return array($this->DOI, $this->ARK, $this->Handle, $this->UUID);
  }

  private function get_pid_fields(): array
  {
    $fields = array();
    foreach ($this->get_pid_types() as $item) {
      $fields[] = 'persistent_item_new_' . $item;
    }
    return $fields;
  }

  private function get_pid_buttons(): array
  {
    $buttons = array();
    foreach ($this->get_pid_fields() as $item) {
      $buttons[$item] = str_replace('persistent_item_new_', 'persistent_minter_button_', $item);
    }
    return $buttons;
  }

  //Build new, existing persistent identifier fields and 'Provision' button. Setup their state relationship.
  private function buildPersistentItemFields(string $pid_type, array &$element, $index, $field_name, $delta, $label, $is_new_pid, $is_doi_finalized, $persistent_item_orig)
  {
    $persistent_item_new = 'persistent_item_new_' . $pid_type;
    //$persistent_item_new_name = $persistent_item_new . "_" . $index;

    $persistent_item = 'persistent_item_' . $pid_type;
    //$persistent_item_name = $persistent_item . "_" . $index;

    $persistent_minter_button = 'persistent_minter_button_' . $pid_type;
    $persistent_minter_button_name = $persistent_minter_button . "_" . $index;

    $persistent_finalize_button = 'persistent_finalize_button_' . $pid_type;
    $persistent_finalize_button_name = $persistent_finalize_button . "_" . $index;

    //$p_new_checkbox = "p_new_checkbox_" . $pid_type;
    //$p_new_checkbox = "p_new_checkbox";
    $pid_url = "pid_url_" . $pid_type;
    //$pid_url_name = $pid_url . "_" . $index;

    //Believe there is a bug in drupal: every time an Ajax callback is called, it loses the states of some elements. So the first click
    //after the Ajax callback cannot produce the correct results
    if (!$this->shouldHideProvisionButton() && $this->isConfigComplete($pid_type)) {

      $element['osp_pid_field_set_detail'][$persistent_item_new] = array(
        '#title' => $this->t('New Persistent Identifier'),
        '#type' => 'textfield',
        '#maxlength' => 222,
        '#element_validate' => [
          [$this, 'validate_new_value'],
        ],
        '#attributes' => [
          "readonly" => "readonly",
        ],
        '#default_value' => $is_new_pid? $persistent_item_orig: "",
        '#states' => [
          'invisible' => [
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
            'or',
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => FALSE],  ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => 'UUID']],
          ],
          'disabled' => [
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => FALSE]],
            /*
            [':input[id="p_pid_type"]' => ['!value' => $pid_type]],
            'or',
            [":input[id=$p_new_checkbox]" => ['checked' => FALSE],  ':input[id="p_pid_type"]' => ['!value' => 'UUID']], */
          ],
        ],
      );
    }
    else
    {
      if (!$this->isConfigComplete($pid_type))
      {
        $element['osp_pid_field_set_detail'][$persistent_item_new] = array(
          '#title' => $this->t('New Persistent Identifier'),
          '#type' => 'textfield',
          '#maxlength' => 222,
          '#default_value' => 'Disabled because of bad configuration',
          '#attributes' => [
            "readonly" => "readonly",
            "disabled" => TRUE,
          ],
          '#states' => [
            'invisible' => [
              [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
              'or',
              [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => FALSE],  ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => 'UUID']],
            ],
          ],
        );
      };

    }

    $element['osp_pid_field_set_detail'][$persistent_item] = array(
      '#title' => $this->t('Existing Persistent Identifier'),
      '#type' => 'textfield',
      '#maxlength' => 222,
      '#element_validate' => [
        [$this, 'validate'],
      ],
      '#default_value' => $is_new_pid? "" : $persistent_item_orig,
      '#states' => [
        'invisible' => [
          [
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
            'or',
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => TRUE]],
            'or',
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['value' => $this->UUID]],
          ],
        ],
        'disabled' => [
          /*
          [':input[id="p_pid_type"]' => [['!value' => $pid_type], ['value' => $this->UUID]] ],
          'or',
          [":input[id=$p_new_checkbox]" => ['checked' => TRUE]],
          */
          /*
          'or',
          [':input[id="p_pid_type"]' => ['value' => $this->UUID]],
          */
          [
            ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => TRUE],
          ],
        ],
      ],
    );

    if (!$this->shouldHideExistingPIDURL())
      $element['osp_pid_field_set_detail'][$pid_url] = array(
        '#title' => $this->t('Existing Persistent Identifier URL'),
        '#type' => 'url',
        '#maxlength' => 1024,
        '#states' => [
          'invisible' => [
            [
              [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
              'or',
              [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => TRUE]],
              'or',
              [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['value' => $this->UUID]],
              /*
              [':input[id="p_pid_type"]' =>
                [
                  ['!value' => 'UUID'],
                  'and',
                  ['value' => $pid_type],
                ]
              ]*/
            ],
          ],
          'disabled' => [
            [
              ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => TRUE],
            ],
          ],
        ],
      );

    if (!$this->shouldHideProvisionButton() && $this->isConfigComplete($pid_type)) {
      \Drupal::logger('persistent_fields')->info("buildPersistentItemFields() field_name is persistent_minter_button=" . $persistent_minter_button . ", wrapper = pid-set-data-wrapper-$index" );
      $element['osp_pid_field_set_detail'][$persistent_minter_button] = array(
        '#value' => $this->t('Provision' . ' ' . $pid_type . ' for ' . $label),
        '#type' => 'button',
        '#attributes' => [
          "name" => "$persistent_minter_button_name",
	  'disabled' => $is_new_pid && !empty($persistent_item_orig)
        ],
        '#ajax' => [
          'callback' => [$this, 'myAjaxCallback'],
          'event' => 'click',
          'wrapper' => "pid-set-data-wrapper-$index"
        ],
        '#states' => [
          'invisible' => [
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
            'or',
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => FALSE],  ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => 'UUID']],
            /*
            [
              ":input[id=$p_new_checkbox]" => ['checked' => FALSE],
            ], */
            //[':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]]
            /*
            'or',
            [
              ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['value' => 'UUID'],
            ],*/
          ],/*
          'enabled' => [
            [':input[id="$persistent_item_new_id"]' => ['value' => '']],
          ],*/
        ],
      );

    }

    if (!$this->shouldHideProvisionButton() && $this->isConfigComplete($pid_type) && $pid_type == "DOI") {
      \Drupal::logger('persistent_fields')->info("buildPersistentItemFields() field_name is persistent_finalize_button=" . $persistent_finalize_button . ", wrapper = pid-set-data-wrapper-$index" );
      $element['osp_pid_field_set_detail'][$persistent_finalize_button] = array(
        '#value' => $this->t('Finalize '  . $pid_type . ' for ' . $label),
        '#type' => 'button',
        '#attributes' => [
          "name" => "$persistent_finalize_button_name",
	  'disabled' => $is_doi_finalized,
        ],
        '#ajax' => [
          'callback' => [$this, 'finalizeCallback'],
          'event' => 'click',
          'wrapper' => "pid-set-data-wrapper-$index"
        ],
        '#states' => [
          'invisible' => [
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]],
            'or',
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][persistent_new_checkbox]"]' => ['checked' => FALSE],  ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => 'UUID']],
            /*
            [
              ":input[id=$p_new_checkbox]" => ['checked' => FALSE],
            ], */
            [':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['!value' => $pid_type]]
            /*
            'or',
            [
              ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['value' => 'UUID'],
            ],*/
          ],/*
          'enabled' => [
            [':input[id="$persistent_item_new_id"]' => ['value' => '']],
          ],*/
        ],
      );

    }
  }


  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state)
  {
    $field_name = $this->fieldDefinition->getName();
    $label = $this->fieldDefinition->getLabel();

    $index = $field_name . "_" . Html::getUniqueId('opfs');

    $osp_pid_field_set_detail = 'osp_pid_field_set_detail' . '_' . $index;
    $element['osp_pid_field_set_detail'] = [
      '#type' => 'details',
      '#title' => $this->t($this->fieldDefinition->getLabel()),
      '#open' => TRUE,
      '#attributes' => [
        "id" => "pid-set-data-wrapper-$index",
      ],
      '#prefix' => '<div id="div-pid-set-data-wrapper-' . $index . '">',
      '#suffix' => '</div>',
    ];

    //$pid_type = 'DOI';
    $pid_type = isset($items[$delta]->pid_type) ? $items[$delta]->pid_type : 'DOI';
    $element['osp_pid_field_set_detail']['doi_finalized'] = array(
      '#type' => 'hidden',
      '#value' => 'no',
    );
    $element['osp_pid_field_set_detail']['pid_type'] = array(
      '#type' => 'select',
      '#title' => $this->t("Please select a persistent identifier type"),
      '#options' => array(
        'DOI' => $this->t('DOI'),
        'ARK' => $this->t('ARK'),
        'Handle' => $this->t('Handle'),
        //'UUID' => $this->t('UUID'),
      ),
      '#default_value' => $pid_type,
    );

    $is_new_pid = isset($items[$delta]->persistent_item_root) && $items[$delta]->persistent_item_root == 'yes';
    $is_doi_finalized = isset($items[$delta]->persistent_item_parent) && $items[$delta]->persistent_item_parent == 'yes';
    if (!$this->shouldHideProvisionButton()) {
      $element['osp_pid_field_set_detail']['persistent_new_checkbox'] = array(
        '#title' => $this->t('New Persistent Identifier'),
        '#type' => 'checkbox',
        '#default_value' => $is_new_pid,
        '#states' => [
          'invisible' => [
            [
              ':input[name="'.$field_name.'['.$delta.'][osp_pid_field_set_detail][pid_type]"]' => ['value' => 'UUID'],
            ],
          ],
        ],
      );
    }
    $persistent_item = isset($items[$delta]->persistent_item)?  $items[$delta]->persistent_item : '';
    $this->buildPersistentItemFields('DOI', $element, $index, $field_name, $delta, $label, $is_new_pid, $is_doi_finalized, $pid_type == 'DOI'? $persistent_item : '' );
    $this->buildPersistentItemFields('ARK', $element, $index, $field_name, $delta, $label, $is_new_pid, $is_doi_finalized, $pid_type == 'ARK'? $persistent_item : '');
    $this->buildPersistentItemFields('Handle', $element, $index, $field_name, $delta, $label, $is_new_pid, $is_doi_finalized, $pid_type == 'Handle'? $persistent_item : '');
    //$this->buildPersistentItemFields('UUID', $element, $index, $field_name, $delta);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state)
  {
    //ksm($form_state);

    $doi_datacite_api_endpoint = \Drupal::configFactory()->getEditable('doi_datacite.settings')->get('doi_datacite_api_endpoint');
    $hdl_handle_api_endpoint = \Drupal::configFactory()->getEditable('hdl.settings')->get('hdl_handle_api_endpoint');


    $url_prefix_mapping = ['New' => ['DOI' =>$doi_datacite_api_endpoint,  'ARK' => 'https://n2t.net/', 'Handle' => $hdl_handle_api_endpoint, 'UUID' => ''],
      'Existing' => ['DOI' => 'https://doi.org/', 'ARK' => 'https://n2t.net/', 'Handle' => 'https://hdl.handle.net/', 'UUID' => '' ]];

    $this->setURLFromFormState($form_state);
    foreach ($values as $delta => $val) {
      $value = $val['osp_pid_field_set_detail'];

      if (empty($value['pid_type']))
        continue;

      $pid_type = $value['pid_type'];
      $values[$delta]['pid_type'] = $pid_type;

      $existing_pid = 'persistent_item_'. $pid_type;
      $new_pid = 'persistent_item_new_'. $pid_type;
      $pid_url = 'pid_url_' . $pid_type;

      $new_or_existing_pid = $value['persistent_new_checkbox'];

      if (key_exists($existing_pid, $value) && $value[$existing_pid] === '' &&
          key_exists($new_pid, $value) && $value[$new_pid] === '') {
        $values[$delta]['persistent_item'] = NULL;
      }

      if (!$new_or_existing_pid)
      if (!empty($value[$existing_pid])) {
        $values[$delta]['persistent_item'] = $value[$existing_pid];
        $values[$delta]['url_prefix'] = $url_prefix_mapping['Existing'][$pid_type];
        $values[$delta]['persistent_item_root'] = 'no';

        if (key_exists($pid_url, $value))
          $values[$delta]['pid_url'] = $value[$pid_url];
        else
          $values[$delta]['pid_url'] = NULL;
      }

      if ($new_or_existing_pid)
      if (!key_exists($existing_pid, $value) || empty($value[$existing_pid])) {
        if (!empty($value[$new_pid])) {
          $values[$delta]['persistent_item'] = $value[$new_pid];
          $values[$delta]['url_prefix'] = $url_prefix_mapping['New'][$pid_type];
          $values[$delta]['persistent_item_root'] = 'yes';
        }
        else
          $values[$delta]['persistent_item'] = NULL;
        $values[$delta]['pid_url'] = $this->url;
      }
      if ($value['doi_finalized'])
        $values[$delta]['persistent_item_parent'] = 'yes';
      else
        $values[$delta]['persistent_item_parent'] = NULL;
      /*
      if (!empty($value['persistent_item_root_hidden']))
        $values[$delta]['persistent_item_root'] = $value['persistent_item_root_hidden'];

      if (!empty($value['persistent_item_parent_hidden']))
        $values[$delta]['persistent_item_parent'] = $value['persistent_item_parent_hidden'];
      */
    }

    return $values;
  }

  public static function defaultSettings()
  {
    return array_merge(
      [
        'hidePersistentMinterButton' => FALSE,
        'hideExistingPIDURL' => TRUE,
        'urlPrefix' => '',

      ],
      parent::defaultSettings());
  }

  public function settingsForm(array $form, FormStateInterface $form_state)
  {

    $this->sanitizeSettings();
    $elements['hidePersistentMinterButton'] = [
      '#title' => $this->t('Hide Provision Button'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hidePersistentMinterButton'),
    ];
    $elements['hideExistingPIDURL'] = [
      '#title' => $this->t('Hide Existing Persistent Identifier URL'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hideExistingPIDURL'),
    ];
    return $elements;

  }

  public function settingsSummary()
  {
    $summary = [];

    if ($this->getSetting('hidePersistentMinterButton')) {
      $summary[] = 'Hidden: Provision Button ';
    } else {
      $summary[] = 'Shown: Provision Button ';
    }
    if ($this->getSetting('hideExistingPIDURL')) {
      $summary[] = 'Hidden: Existing Persistent Identifier URL ';
    } else {
      $summary[] = 'Shown: Existing Persistent Identifier URL ';
    }
    return $summary;
  }

  protected function sanitizeSettings()
  {
    $this->setSetting('hidePersistentMinterButton', boolval($this->getSetting('hidePersistentMinterButton')));
    $this->setSetting('hideExistingPIDURL', boolval($this->getSetting('hideExistingPIDURL')));
    $this->setSetting('urlPrefix', $this->getSetting('urlPrefix'));
  }
}

